"""Shortcut aliases for the Align class.

The names specify the "anchor" point. For example, "BOTTOM" means to align the bottom of
the object to the origin.
"""

import build123d as bd

TOP = (bd.Align.CENTER, bd.Align.CENTER, bd.Align.MAX)
BOTTOM = (bd.Align.CENTER, bd.Align.CENTER, bd.Align.MIN)
FRONT = (bd.Align.CENTER, bd.Align.MIN, bd.Align.CENTER)
BACK = (bd.Align.CENTER, bd.Align.MAX, bd.Align.CENTER)
LEFT = (bd.Align.MIN, bd.Align.CENTER, bd.Align.CENTER)
RIGHT = (bd.Align.MAX, bd.Align.CENTER, bd.Align.CENTER)

CENTER = (bd.Align.CENTER, bd.Align.CENTER, bd.Align.CENTER)

TOP_LEFT = (bd.Align.MIN, bd.Align.MIN, bd.Align.MAX)
TOP_RIGHT = (bd.Align.MAX, bd.Align.MIN, bd.Align.MAX)
BOTTOM_LEFT = (bd.Align.MIN, bd.Align.MAX, bd.Align.MAX)
BOTTOM_RIGHT = (bd.Align.MAX, bd.Align.MAX, bd.Align.MAX)

FRONT_LEFT = (bd.Align.MIN, bd.Align.MIN, bd.Align.CENTER)
FRONT_RIGHT = (bd.Align.MAX, bd.Align.MIN, bd.Align.CENTER)
BACK_LEFT = (bd.Align.MIN, bd.Align.MAX, bd.Align.CENTER)
BACK_RIGHT = (bd.Align.MAX, bd.Align.MAX, bd.Align.CENTER)



"""Shortcut aliases for rotations.

Consider a regular Cylinder (Z-cylinder) with its axis along the Z-axis. Applying
this rotation results in the cylinder pointing in this given direction.
"""

POS_X = (0, 90, 0)
NEG_X = (0, -90, 0)
POS_Y = (-90, 0, 0)
NEG_Y = (90, 0, 0)
POS_Z = (0, 0, 0)
NEG_Z = (180, 0, 0)



"""Fetch faces by name from a build123d Part object."""

from typing import Literal

import build123d as bd


def top_face_of(part: bd.Part) -> bd.Face:
    """Return the top face of the given Part object."""
    return part.faces().sort_by(bd.Axis.Z)[-1]


def bottom_face_of(part: bd.Part) -> bd.Face:
    """Return the bottom face of the given Part object."""
    return part.faces().sort_by(bd.Axis.Z)[0]


def left_face_of(part: bd.Part) -> bd.Face:
    """Return the left face of the given Part object."""
    return part.faces().sort_by(bd.Axis.X)[0]


def right_face_of(part: bd.Part) -> bd.Face:
    """Return the right face of the given Part object."""
    return part.faces().sort_by(bd.Axis.X)[-1]


def front_face_of(part: bd.Part) -> bd.Face:
    """Return the front face of the given Part object."""
    return part.faces().sort_by(bd.Axis.Y)[0]


def back_face_of(part: bd.Part) -> bd.Face:
    """Return the back face of the given Part object."""
    return part.faces().sort_by(bd.Axis.Y)[-1]


def get_face_by_name(
    part: bd.Part,
    face_name: Literal["top", "bottom", "left", "right", "front", "back"],
) -> bd.Face:
    """Return the face of the given Part object with the given name.

    Raises:
        ValueError: If the face name is invalid.

    Returns:
        bd.Face: The requested face.

    """
    if face_name == "top":
        return top_face_of(part)
    if face_name == "bottom":
        return bottom_face_of(part)
    if face_name == "left":
        return left_face_of(part)
    if face_name == "right":
        return right_face_of(part)
    if face_name == "front":
        return front_face_of(part)
    if face_name == "back":
        return back_face_of(part)

    msg = f"Invalid face name: {face_name}"
    raise ValueError(msg)



"""Extensions, tools, and shortcuts to make modelling with Build123d easier."""

from . import align, fetch_faces, rotation
from .ci_show import show
from .fetch_faces import (
    back_face_of,
    bottom_face_of,
    front_face_of,
    left_face_of,
    right_face_of,
    top_face_of,
)

__VERSION__ = "0.1.2"

__all__ = [
    "align",
    "rotation",
    "show",
    "fetch_faces",
    "top_face_of",
    "bottom_face_of",
    "left_face_of",
    "right_face_of",
    "front_face_of",
    "back_face_of",
]



"""Create a version of the `show()` function which runs silently in a CI environment."""

# pyright: reportMissingTypeStubs=information, reportUnknownMemberType=information

import os

import build123d as bd

if os.getenv("CI"):

    def show(*args: object) -> bd.Part:
        """Do nothing (dummy function) to skip showing the CAD model in CI."""
        if not isinstance(args[0], bd.Part):
            msg = "The first argument must be a Part object."
            raise TypeError(msg)

        return args[0]
else:
    import ocp_vscode

    def show(*args: object) -> bd.Part:
        """Show the CAD model in the CAD viewer."""
        if not isinstance(args[0], bd.Part):
            msg = "The first argument must be a Part object."
            raise TypeError(msg)

        ocp_vscode.show(*args)
        return args[0]



"""Tests for the `rotation` module."""

# pyright: reportCallIssue=information

import build123d as bd

import build123d_ease as bde


def test_pos_x_rotation() -> None:
    """Applying the POS_X rotation to a cylinder will make it point that direction."""
    p = bd.Part() + bd.Cylinder(
        radius=10,
        height=10,
        rotation=bde.rotation.POS_X,
        align=bde.align.BOTTOM,  # Align bottom happens **before** the rotation.
    )

    part_pos_x = p & bd.Box(100, 100, 100, align=bde.align.LEFT)
    assert p.volume == part_pos_x.volume


def test_neg_x_rotation() -> None:
    """Applying the NEG_X rotation to a cylinder will make it point that direction."""
    p = bd.Part() + bd.Cylinder(
        radius=10,
        height=10,
        rotation=bde.rotation.NEG_X,
        align=bde.align.BOTTOM,  # Align bottom happens **before** the rotation.
    )

    part_neg_x = p & bd.Box(100, 100, 100, align=bde.align.RIGHT)
    assert p.volume == part_neg_x.volume


def test_pos_y_rotation() -> None:
    """Applying the POS_Y rotation to a cylinder will make it point that direction."""
    p = bd.Part() + bd.Cylinder(
        radius=10,
        height=10,
        rotation=bde.rotation.POS_Y,
        align=bde.align.BOTTOM,  # Align bottom happens **before** the rotation.
    )

    part_pos_y = p & bd.Box(100, 100, 100, align=bde.align.FRONT)
    assert p.volume == part_pos_y.volume


def test_neg_y_rotation() -> None:
    """Applying the NEG_Y rotation to a cylinder will make it point that direction."""
    p = bd.Part() + bd.Cylinder(
        radius=10,
        height=10,
        rotation=bde.rotation.NEG_Y,
        align=bde.align.BOTTOM,  # Align bottom happens **before** the rotation.
    )

    part_neg_y = p & bd.Box(100, 100, 100, align=bde.align.BACK)
    assert p.volume == part_neg_y.volume


# We will just trust that POS_Z (no rotation) and NEG_Z (a 180-degree rotation) work as
# expected, without tests.
